import argparse
import sys

from docxlatex import Document


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "ip",
        type=str,
        help="An absolute or relative path to the input .docx file",
    )
    parser.add_argument(
        "--op",
        type=str,
        help="An absolute or relative path to the output file",
        default=None,
    )
    parser.add_argument(
        "--xml",
        action="store_true",
        help="Dump the document's XML instead of converting to text",
    )
    parser.add_argument(
        "-l",
        action="store_true",
        default=False,
        help='Specifies that the document has been converted to "Linear" format',
    )
    args = parser.parse_args()

    try:
        doc = Document(args.ip)
        if args.xml:
            if args.op is None:
                print(doc.pprint_xml())
            else:
                with open(args.op, "w", encoding="utf-8") as f:
                    f.write(doc.get_xml())
        else:
            if args.op is None:
                print(doc.get_text(linear_format=args.l))
            else:
                with open(args.op, "w") as f:
                    f.write(doc.get_text(linear_format=args.l))
    except Exception as e:
        print(e)
        sys.exit(1)



"""
Postprocessing functions for cleaning up latex equations in linear format which don't give valid LaTeX.
"""
import re

clean_exps = {
    r"\\degf": "&deg;F",
    r"\\degc": "&deg;C",
    r"(\\cbrt)(\w+)": r"\\sqrt[3]{\2}",
    r"(\\qdrt)(\w+)": r"\\sqrt[4]{\2}",
    r"\\sfrac": r"\\frac",
    r"(\\o[i]+nt)(\w+)": r"\1{\2}",
    r"\\bullet(\w+)": r"\\bullet \1",
    r"\\sum([a-zA-Z0-9]+)": r"\\sum{\1}",
    r"\\prod([a-zA-Z0-9]+)": r"\\prod{\1}",
    r"\\amalg([a-zA-Z0-9]+)": r"\\amalg{\1}",
    r"\\bigcup([a-zA-Z0-9]+)": r"\\bigcup{\1}",
    r"\\bigcap([a-zA-Z0-9]+)": r"\\bigcap{\1}",
    r"\\bigvee([a-zA-Z0-9]+)": r"\\bigvee{\1}",
    r"\\bigwedge([a-zA-Z0-9]+)": r"\\bigwedge{\1}",
    r"\\lfloor([a-zA-Z0-9]+)": r"\\lfloor{\1}",
    r"\\lceil([a-zA-Z0-9]+)": r"\\lceil{\1}",
    r"\\lim\\below\{(.+)\}\{(.+)\}": r"\\lim_{\1}{\2}",
    r"\\min\\below\{(.+)\}\{(.+)\}": r"\\min_{\1}{\2}",
    r"\\max\\below\{(.+)\}\{(.+)\}": r"\\max_{\1}{\2}",
}


def clean_exp(exp):
    """
    Takes in a linear expression and converts known invalid LaTeX equations to valid LaTeX
    :param exp:str - An equation in invalid syntax
    :return :str - A valid equation
    """
    for e in clean_exps:
        exp = re.sub(e, clean_exps[e], exp)
    return exp



import unittest

from docxlatex import Document


class SimpleTextExtractionTestCase(unittest.TestCase):
    def test_simple_text_extraction(self):
        doc = Document("./docx/simple-text.docx")
        text = doc.get_text().strip()
        expected = "Test Document\n\nThis is paragraph 1."
        self.assertEqual(text, expected)

    def test_simple_equation_extraction_linear_format(self):
        doc = Document("./docx/simple-equation-linear-format.docx")
        text = doc.get_text().strip()
        expected = "$ a^2+b^2=c^2 $"
        self.assertEqual(text, expected)


if __name__ == "__main__":
    unittest.main()



from docxlatex import Document
import unittest


class TestSymbols(unittest.TestCase):
    def test_operators(self):
        """Test extraction of operators and operator structures found in the Equation > Operator menu."""
        text = Document("./docx/operators.docx").get_text(linear_format=False).strip()
        self.assertEqual(
            text,
            "$ ∶= == += -=\\stackrel{\\tiny def}{=} \\stackrel{\\tiny m}{=} \\triangleq $",
        )
        text = (
            Document("./docx/operator_structures.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual(
            text,
            "$ \\underset{a}{\\leftarrow} \\underset{a}{\\rightarrow} \\overset{a}{\\leftarrow} \\overset{a}{\\rightarrow} \\underset{a}{\\Leftarrow} \\underset{a}{\\Rightarrow} \\overset{a}{\\Leftarrow} \\overset{a}{\\Rightarrow} \\underset{a}{\\leftrightarrow} \\overset{a}{\\leftrightarrow} \\underset{a}{\\Leftrightarrow} \\overset{a}{\\Leftrightarrow} $",
        )

    def test_basic_math_symbols(self):
        text = (
            Document("./docx/basic_math_operators.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual(
            text,
            "$ ±\\infty =≠~×÷!∝\\lt ≪\\gt ≫\\leq \\geq ∓≅≈≡∀∁∂√∛∜∪∩∅%°℉℃∆∇∃∄∈∋←↑→↓↔∴+-¬αβγδεϵθϑμπρστφω*∙⋮⋯⋰⋱ℵℶ∎ $",
        )

    def test_greek_letters(self):
        text = (
            Document("./docx/greek_letters.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual(
            text,
            "$ αβγδεϵζηθϑικλμνξοπϖρϱσςτυφϕχψω $\n\n$ ΑΒΓΔΕΖΗΘΙΚΛΜΝΞΟΠΡΣΤΥΦΧΨΩ $",
        )

    def test_letter_like_symbols(self):
        # TODO: Add support for different scripts of all letter-like symbols
        text = (
            Document("./docx/letter_like_symbols.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual(
            text,
            "$ ∀∁C∂ðℇϜℲgHHhℏ℩ıIjϰLlN℘QRRRZ℧ÅB℮E∃∄FMoℵℶℷℸ $",
        )

    def test_common_binary_operators(self):
        text = (
            Document("./docx/common_binary_operators.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual(
            text,
            "$ +-÷×±∓∝∕*∘∙⋅∩∪⊎⊓⊔∧∨ $",
        )

    def test_common_relational_operators(self):
        text = (
            Document("./docx/common_relational_operators.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual(
            text,
            "$ =≠\\lt \\gt \\leq \\geq ≮≰≯≱≡∼≃≈≅≢≄≉≇∝≪≫∈∋∉⊂⊃⊆⊇≺≻≼≽⊏⊐⊑⊒∥⊥⊢⊣⋈≍ $",
        )

    def test_basic_nary_operators(self):
        text = (
            Document("./docx/basic_nary_operators.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual(
            text,
            "$ ∑∫∬∭∮∯∰∱∲∳∏∐⋂⋃⋀⋁⨀⨂⨁⨄⨃ $",
        )

    def test_advanced_binary_operators(self):
        text = (
            Document("./docx/advanced_binary_operators.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual(
            text,
            "$ ∔∸∖⋒⋓⊟⊠⊡⊞⋇⋉⋊⋋⋌⋏⋎⊝⊺⊕⊖⊗⊘⊙⊛⊚†‡⋆⋄≀△⋀⋁⨀⨂⨁⨅⨆⨄⨃ $",
        )

    def test_advanced_relational_operators(self):
        text = (
            Document("./docx/advanced_relational_operators.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual(
            text,
            "$ ∴∵⋘⋙≦≧≲≳⋖⋗≶⋚≷⋛≑≒≓∽≊⋍≼≽⋞⋟≾≿⋜⋝⊆⊇⊲⊳⊴⊵⊨⋐⋑⊏⊐⊩⊪≖≗≜≏≎∝≬⋔≐⋈ $",
        )


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from sphinx.errors import ExtensionError


def add_ga_javascript(app, pagename, templatename, context, doctree):
    if not app.config.googleanalytics_enabled:
        return

    metatags = context.get("metatags", "")
    metatags += (
        """
    <!-- Google tag (gtag.js) -->
    <script async src="https://www.googletagmanager.com/gtag/js?id=%s"></script>"""
        % app.config.googleanalytics_id
    )
    metatags += (
        """
    <script>
        window.dataLayer = window.dataLayer || [];
        function gtag(){dataLayer.push(arguments);}
        gtag('js', new Date());
        gtag('config', '%s');
    </script>
    """
        % app.config.googleanalytics_id
    )
    context["metatags"] = metatags


def check_config(app):
    if app.config.googleanalytics_enabled and not app.config.googleanalytics_id:
        raise ExtensionError(
            "'googleanalytics_id' config value must be set for ga statistics to function properly."
        )


def setup(app):
    app.add_config_value("googleanalytics_id", "", "html")
    app.add_config_value("googleanalytics_enabled", True, "html")
    app.connect("html-page-context", add_ga_javascript)
    app.connect("builder-inited", check_config)
    return {
        "version": "0.3",
        "parallel_read_safe": True,
        "parallel_write_safe": True,
    }



import unittest
from docxlatex import Document


class TestTags(unittest.TestCase):
    """
    Test the extraction of OMML tags independently
    with very simple nesting or no nesting of tags.
    """

    def test_r(self):
        text = Document("./docx/tags/r/r.docx").get_text(linear_format=False).strip()
        self.assertEqual(text, "$ a+b+c+d $")

    def test_acc(self):
        text = (
            Document("./docx/tags/acc/dot.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\dot{a} $", text)
        text = (
            Document("./docx/tags/acc/ddot.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\ddot{a} $", text)
        text = (
            Document("./docx/tags/acc/tdot.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\dddot{a} $", text)
        text = (
            Document("./docx/tags/acc/hat.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\hat{a} $", text)
        text = (
            Document("./docx/tags/acc/check.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\check{a} $", text)
        text = (
            Document("./docx/tags/acc/tilde.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\tilde{a} $", text)
        text = (
            Document("./docx/tags/acc/acute.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\acute{a} $", text)
        text = (
            Document("./docx/tags/acc/grave.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\grave{a} $", text)
        text = (
            Document("./docx/tags/acc/breve.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\breve{a} $", text)
        text = (
            Document("./docx/tags/acc/bar.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\bar{a} $", text)
        text = (
            Document("./docx/tags/acc/dbar.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\overline{\\overline{a}} $", text)
        text = (
            Document("./docx/tags/acc/overleftarrow.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\overleftarrow{a} $", text)
        text = (
            Document("./docx/tags/acc/overrightarrow.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\overrightarrow{a} $", text)
        text = (
            Document("./docx/tags/acc/overleftrightarrow.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\overset\\leftrightarrow{a} $", text)
        text = (
            Document("./docx/tags/acc/leftharpoon.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\overset\\leftharpoonup{a} $", text)
        text = (
            Document("./docx/tags/acc/rightharpoon.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\overset\\rightharpoonup{a} $", text)
        text = (
            Document("./docx/tags/acc/tilde-bar.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\bar{\\tilde{a}} $", text)

    def test_border_box(self):
        text = (
            Document("./docx/tags/borderBox/borderBox.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\boxed{a} $", text)
        text = (
            Document("./docx/tags/borderBox/boxedFormula.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\boxed{boxed formula} $", text)

    def test_bar(self):
        text = (
            Document("./docx/tags/bar/overline.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\overline{asdf} $", text)
        # Section 22.1.2.8 of the spec says that the pos child of the barPr element
        # must have value either top or bot, but the test document does not have a pos child.
        # Therefore, the default value is top and docxlatex will return an overline instead of
        # an underline.
        text = (
            Document("./docx/tags/bar/underline.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\overline{asdf} $", text)
        text = (
            Document("./docx/tags/bar/overline-tilde.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\overline{\\tilde{a}sdf} $", text)

    def test_group_chr(self):
        text = (
            Document("./docx/tags/groupChr/overbrace.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\overbrace{asdf} $", text)
        text = (
            Document("./docx/tags/groupChr/underbrace.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\underbrace{asdf} $", text)

    def test_f(self):
        text = Document("./docx/tags/f/f.docx").get_text(linear_format=False).strip()
        self.assertEqual("$ \\frac{abc}{def} $", text)
        text = Document("./docx/tags/f/dydx.docx").get_text(linear_format=False).strip()
        self.assertEqual("$ \\frac{dy}{dx} $", text)

    def test_s_sup(self):
        text = (
            Document("./docx/tags/sSup/exponent.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ {a}^{b} $", text)
        text = (
            Document("./docx/tags/sSup/abcdef.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ {abc}^{def} $", text)

    def test_s_sub(self):
        text = (
            Document("./docx/tags/sSub/subscript.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ {a}_{b} $", text)
        text = (
            Document("./docx/tags/sSub/abcdef.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ {abc}_{def} $", text)

    def test_s_sub_sup(self):
        text = (
            Document("./docx/tags/sSubSup/sub_sup.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ {a}_{b}^{c} $", text)
        text = (
            Document("./docx/tags/sSubSup/abcdefghi.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ {abc}_{def}^{ghi} $", text)

    def test_s_pre(self):
        text = (
            Document("./docx/tags/sPre/pre_sub_sup.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ {}^{b}_{a}{c} $", text)

    def test_rad(self):
        text = (
            Document("./docx/tags/rad/sqrt.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\sqrt{a} $", text)
        text = (
            Document("./docx/tags/rad/cube_root.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\sqrt[3]{a} $", text)
        text = (
            Document("./docx/tags/rad/nth_root.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\sqrt[n]{a} $", text)

    def test_nary(self):
        text = (
            Document("./docx/tags/nary/integral.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\int{x dx} $", text)
        text = (
            Document("./docx/tags/nary/integral2.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\int_{0}^{1}{x dx} $", text)
        text = (
            Document("./docx/tags/nary/integral3.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\int_{0}^{1}{x dx} $", text)
        text = (
            Document("./docx/tags/nary/iint.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\iint{xy dxdy} $", text)
        text = (
            Document("./docx/tags/nary/iint2.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\iint_{0}^{1}{xy dxdy} $", text)
        text = (
            Document("./docx/tags/nary/iint3.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\iint_{0}^{1}{xy dxdy} $", text)
        text = (
            Document("./docx/tags/nary/iiint.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\iiint{x dx} $", text)
        text = (
            Document("./docx/tags/nary/iiint2.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\iiint_{0}^{1}{x dx} $", text)
        text = (
            Document("./docx/tags/nary/iiint3.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\iiint_{0}^{1}{x dx} $", text)
        text = (
            Document("./docx/tags/nary/oint.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\oint{x dx} $", text)
        text = (
            Document("./docx/tags/nary/oint2.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\oint_{0}^{1}{x dx} $", text)
        text = (
            Document("./docx/tags/nary/oint3.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\oint_{0}^{1}{x dx} $", text)
        text = (
            Document("./docx/tags/nary/oiint.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\oiint{x dx} $", text)
        text = (
            Document("./docx/tags/nary/oiint2.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\oiint_{0}^{1}{x dx} $", text)
        text = (
            Document("./docx/tags/nary/oiint3.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\oiint_{0}^{1}{x dx} $", text)
        text = (
            Document("./docx/tags/nary/oiiint.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\oiiint{x dx} $", text)
        text = (
            Document("./docx/tags/nary/oiiint2.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\oiiint_{0}^{1}{x dx} $", text)
        text = (
            Document("./docx/tags/nary/oiiint3.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\oiiint_{0}^{1}{x dx} $", text)
        text = (
            Document("./docx/tags/nary/sum.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\sum{i} $", text)
        text = (
            Document("./docx/tags/nary/sum2.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\sum_{0}^{1}{i} $", text)
        text = (
            Document("./docx/tags/nary/sum3.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\sum_{0}^{1}{i} $", text)
        text = (
            Document("./docx/tags/nary/sum4.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\sum_{0}{i} $", text)
        text = (
            Document("./docx/tags/nary/sum5.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\sum_{0}{i} $", text)
        text = (
            Document("./docx/tags/nary/prod.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\prod{i} $", text)
        text = (
            Document("./docx/tags/nary/prod2.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\prod_{0}^{1}{i} $", text)
        text = (
            Document("./docx/tags/nary/prod3.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\prod_{0}^{1}{i} $", text)
        text = (
            Document("./docx/tags/nary/prod4.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\prod_{0}{i} $", text)
        text = (
            Document("./docx/tags/nary/prod5.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\prod_{0}{i} $", text)
        text = (
            Document("./docx/tags/nary/coprod.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\coprod{i} $", text)
        text = (
            Document("./docx/tags/nary/coprod2.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\coprod_{0}^{1}{i} $", text)
        text = (
            Document("./docx/tags/nary/coprod3.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\coprod_{0}^{1}{i} $", text)
        text = (
            Document("./docx/tags/nary/coprod4.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\coprod_{0}{i} $", text)
        text = (
            Document("./docx/tags/nary/coprod5.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\coprod_{0}{i} $", text)
        text = (
            Document("./docx/tags/nary/cup.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\bigcup{i} $", text)
        text = (
            Document("./docx/tags/nary/cup2.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\bigcup_{0}^{1}{i} $", text)
        text = (
            Document("./docx/tags/nary/cup3.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\bigcup_{0}^{1}{i} $", text)
        text = (
            Document("./docx/tags/nary/cup4.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\bigcup_{0}{i} $", text)
        text = (
            Document("./docx/tags/nary/cup5.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\bigcup_{0}{i} $", text)
        text = (
            Document("./docx/tags/nary/cap.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\bigcap{i} $", text)
        text = (
            Document("./docx/tags/nary/cap2.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\bigcap_{0}^{1}{i} $", text)
        text = (
            Document("./docx/tags/nary/cap3.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\bigcap_{0}^{1}{i} $", text)
        text = (
            Document("./docx/tags/nary/cap4.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\bigcap_{0}{i} $", text)
        text = (
            Document("./docx/tags/nary/cap5.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\bigcap_{0}{i} $", text)
        text = (
            Document("./docx/tags/nary/bigvee.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\bigvee{i} $", text)
        text = (
            Document("./docx/tags/nary/bigvee2.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\bigvee_{0}^{1}{i} $", text)
        text = (
            Document("./docx/tags/nary/bigvee3.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\bigvee_{0}^{1}{i} $", text)
        text = (
            Document("./docx/tags/nary/bigvee4.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\bigvee_{0}{i} $", text)
        text = (
            Document("./docx/tags/nary/bigvee5.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\bigvee_{0}{i} $", text)
        text = (
            Document("./docx/tags/nary/bigwedge.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\bigwedge{i} $", text)
        text = (
            Document("./docx/tags/nary/bigwedge2.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\bigwedge_{0}^{1}{i} $", text)
        text = (
            Document("./docx/tags/nary/bigwedge3.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\bigwedge_{0}^{1}{i} $", text)
        text = (
            Document("./docx/tags/nary/bigwedge4.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\bigwedge_{0}{i} $", text)
        text = (
            Document("./docx/tags/nary/bigwedge5.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\bigwedge_{0}{i} $", text)

    def test_d(self):
        text = (
            Document("./docx/tags/d/parenthesis.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\left( asdf \\right) $", text)
        text = (
            Document("./docx/tags/d/square_brackets.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\left[ a b c d \\right] $", text)
        text = (
            Document("./docx/tags/d/square_brackets_2.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\left[ a \\right[ $", text)
        text = (
            Document("./docx/tags/d/square_brackets_3.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\left] a \\right] $", text)
        text = (
            Document("./docx/tags/d/square_brackets_4.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\left] a \\right[ $", text)
        text = (
            Document("./docx/tags/d/curly_brackets.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\left{ a b c d \\right} $", text)
        text = (
            Document("./docx/tags/d/angle_brackets.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\left\\langle a b c d \\right\\rangle $", text)
        text = (
            Document("./docx/tags/d/floor.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\left\\lfloor a b c d \\right\\rfloor $", text)
        text = Document("./docx/tags/d/ceil.docx").get_text(linear_format=False).strip()
        self.assertEqual("$ \\left\\lceil a b c d \\right\\rceil $", text)
        text = Document("./docx/tags/d/abs.docx").get_text(linear_format=False).strip()
        self.assertEqual("$ \\left| a b c d \\right| $", text)
        text = Document("./docx/tags/d/norm.docx").get_text(linear_format=False).strip()
        self.assertEqual("$ \\left\\| a b c d \\right\\| $", text)
        text = (
            Document("./docx/tags/d/double_brackets.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ [\\![ a ]\\!] $", text)
        text = (
            Document("./docx/tags/d/parenthesis_2.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\left( a|b \\right) $", text)
        text = (
            Document("./docx/tags/d/parenthesis_3.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\left( a $", text)
        text = (
            Document("./docx/tags/d/parenthesis_4.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ a \\right) $", text)

    def test_m(self):
        text = (
            Document("./docx/tags/m/simple-matrix.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\begin{pmatrix} 1 & 2 \\\\ 3 & 4 \\end{pmatrix} $", text)
        text = (
            Document("./docx/tags/m/simple-matrix-2.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\begin{bmatrix} 1 & 2 \\\\ 3 & 4 \\end{bmatrix} $", text)
        text = (
            Document("./docx/tags/m/matrix-no-brackets.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\begin{matrix} 1 & 2 \\\\ 3 & 4 \\end{matrix} $", text)
        text = (
            Document("./docx/tags/m/matrix-no-brackets-3x3.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual(
            "$ \\begin{matrix} 1 & 2 & 3 \\\\ 4 & 5 & 6 \\\\ 7 & 8 & 9 \\end{matrix} $",
            text,
        )
        text = (
            Document("./docx/tags/m/determinant.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\begin{vmatrix} 1 & 2 \\\\ 3 & 4 \\end{vmatrix} $", text)
        text = (
            Document("./docx/tags/m/matrix-double-bars.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\begin{Vmatrix} 1 & 2 \\\\ 3 & 4 \\end{Vmatrix} $", text)
        text = (
            Document("./docx/tags/m/sparse-matrix.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual(
            "$ \\begin{pmatrix} 1 & ⋯ & 2 \\\\ ⋮ & ⋱ & ⋮ \\\\ 3 & ⋯ & 4 \\end{pmatrix} $",
            text,
        )

    def test_eq_array(self):
        text = (
            Document("./docx/tags/eqArr/stack.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\begin{cases} a \\\\b \\\\ \\end{cases} $", text)
        text = (
            Document("./docx/tags/eqArr/stack2.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual("$ \\begin{cases} a \\\\b \\\\c \\\\ \\end{cases} $", text)

        text = (
            Document("./docx/tags/eqArr/issue_9.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual(
            "$ f\\left( n \\right)=\\begin{cases} \\left\\langle 0,0 \\right\\rangle,&{χ}_{TH}(n)=1 \\\\\\left\\langle 0,m \\right\\rangle, &otherwise \\\\ \\end{cases} $",
            text,
        )

    def test_func(self):
        text = (
            Document("./docx/tags/func/trig.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual(
            "$ \\sin_{}^{}{x} \\cos_{}^{}{x} \\tan_{}^{}{x} \\csc_{}^{}{x} \\sec_{}^{}{x} \\cot_{}^{}{x} $",
            text,
        )
        text = (
            Document("./docx/tags/func/inverse_trig.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual(
            "$ \\sin_{}^{-1}{x} \\cos_{}^{-1}{x} \\tan_{}^{-1}{x} \\csc_{}^{-1}{x} \\sec_{}^{-1}{x} \\cot_{}^{-1}{x} $",
            text,
        )
        text = (
            Document("./docx/tags/func/hyperbolic.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual(
            "$ \\sinh_{}^{}{x} \\cosh_{}^{}{x} \\tanh_{}^{}{x} \\operatorname{csch}_{}^{}{x} \\operatorname{sech}_{}^{}{x} \\coth_{}^{}{x} $",
            text,
        )
        text = (
            Document("./docx/tags/func/inverse_hyperbolic.docx")
            .get_text(linear_format=False)
            .strip()
        )
        self.assertEqual(
            "$ \\sinh_{}^{-1}{x} \\cosh_{}^{-1}{x} \\tanh_{}^{-1}{x} \\operatorname{csch}_{}^{-1}{x} \\operatorname{sech}_{}^{-1}{x} \\coth_{}^{-1}{x} $",
            text,
        )
        text = (
            Document("./docx/tags/func/log.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\log_{a}^{}{b} $", text)
        text = (
            Document("./docx/tags/func/log2.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\log_{}^{}{a} $", text)
        text = (
            Document("./docx/tags/func/log3.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\ln_{}^{}{a} $", text)
        text = (
            Document("./docx/tags/func/lim.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\lim\\limits_{a}^{}{b} $", text)
        text = (
            Document("./docx/tags/func/max.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\max\\limits_{a}^{}{b} $", text)
        text = (
            Document("./docx/tags/func/min.docx").get_text(linear_format=False).strip()
        )
        self.assertEqual("$ \\min\\limits_{a}^{}{b} $", text)


if __name__ == "__main__":
    unittest.main()



import unittest
from docxlatex import Document


class TestEquations(unittest.TestCase):
    def test_equations(self):
        text = Document("./docx/binomial-theorem.docx").get_text().strip()
        self.assertEqual(
            text,
            "$ {\\left( x+a \\right)}^{n}=\\sum_{k=0}^{n}{\\left( \\genfrac{}{}{0pt}{}{n}{k} \\right){x}^{k}{a}^{n-k}} $",
        )

        text = Document("./docx/fourier_series.docx").get_text().strip()
        self.assertEqual(
            text,
            "$ f\\left( x \\right)={a}_{0}+\\sum_{n=1}^{\\infty }{\\left( {a}_{n}\\cos_{}^{}{\\frac{nπx}{L}}+{b}_{n}\\sin_{}^{}{\\frac{nπx}{L}} \\right)} $",
        )

        text = Document("./docx/taylor_expansion.docx").get_text().strip()
        self.assertEqual(
            text,
            "$ {e}^{x}=1+\\frac{x}{1!}+\\frac{{x}^{2}}{2!}+\\frac{{x}^{3}}{3!}+…, -\\infty \\lt x\\lt \\infty  $",
        )


if __name__ == "__main__":
    unittest.main()



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information


# ! To build docs, run `make html` in the docs_src/ directory.


project = "docxlatex"
copyright = "2025, Hrushikesh Vaidya (@hrushikeshrv)"
author = "Hrushikesh Vaidya (@hrushikeshrv)"
release = "v1.2.3"

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    "sphinxcontrib.googleanalytics",
]
googleanalytics_id = "G-TGM5F6WJNV"

templates_path = ["_templates"]
exclude_patterns = []


# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = "furo"
html_static_path = ["_static"]
html_theme_options = {
    "sidebar_hide_name": False,
    "navigation_with_keys": True,
}



from xml.etree.cElementTree import Element

from docxlatex.parser.utils import qn


class OMMLParser:
    """
    Parser class for reading OMML and converting it into LaTeX.
    """

    def parse(self, root: Element) -> str:
        """
        Parses an m:oMath OMML tag into LaTeX.
        :param root: An m:oMath OMML tag
        :return: The LaTeX representation of the OMML input
        """
        text = ""
        try:
            if root.tag == qn("m:t"):
                return self.parse_t(root)
            for child in root:
                if child.tag in self.parsers:
                    text += self.parsers[child.tag](self, child)
        except AttributeError:
            # In case of missing attributes on OMML tags,
            # we return an empty string (ref:issue_14)
            return ""
        return text

    def parse_e(self, root: Element) -> str:
        text = ""
        for child in root:
            text += self.parse(child)
        return text

    def parse_r(self, root: Element) -> str:
        # TODO: Add support for m:rPr and m:scr to support different character styles
        #    For now, we just parse the text content of m:r
        text = ""
        for child in root:
            text += self.parse(child)
        return text

    def parse_t(self, root: Element):
        symbol_map = {
            "≜": "\\triangleq",
            "≝": "\\stackrel{\\tiny def}{=}",
            "≞": "\\stackrel{\\tiny m}{=}",
        }
        replacements = {
            "&lt;": "\\lt ",
            "&gt;": "\\gt ",
            "&le;": "\\leq ",
            "&ge;": "\\geq ",
            "∞": "\\infty ",
            "<": "\\lt ",
            ">": "\\gt ",
            "≤": "\\leq ",
            "≥": "\\geq ",
        }
        text = root.text.split()
        if not text:
            return " "
        for i, t in enumerate(text):
            if t in symbol_map:
                text[i] = symbol_map[t]
        for key, value in replacements.items():
            for i, t in enumerate(text):
                text[i] = t.replace(key, value)
        return " ".join(text)

    def parse_acc(self, root: Element) -> str:
        character_map = {
            768: "\\grave",
            769: "\\acute",
            770: "\\hat",
            771: "\\tilde",
            773: "\\bar",
            774: "\\breve",
            775: "\\dot",
            776: "\\ddot",
            780: "\\check",
            831: "\\overline{\\overline",
            8400: "\\overset\\leftharpoonup",
            8401: "\\overset\\rightharpoonup",
            8406: "\\overleftarrow",
            8407: "\\overrightarrow",
            8411: "\\dddot",
            8417: "\\overset\\leftrightarrow",
        }
        text = ""
        accent = 770
        for child in root:
            if child.tag == qn("m:accPr"):
                for child2 in child:
                    if child2.tag == qn("m:chr"):
                        accent = ord(child2.attrib.get(qn("m:val")))

        text += character_map[accent] + "{"
        for child in root:
            if child.tag == qn("m:e"):
                text += self.parse(child)
        text += "}"
        if accent == 831:
            text += "}"
        return text

    def parse_bar(self, root: Element) -> str:
        text = "\\overline{"
        for child in root:
            if child.tag == qn("m:barPr"):
                for child2 in child:
                    if child2.tag == qn("m:pos"):
                        if child2.attrib.get(qn("m:val")) == "bot":
                            text = "\\underline{"

        for child in root:
            if child.tag == qn("m:e"):
                text += self.parse(child)
        text += "}"
        return text

    def parse_border_box(self, root: Element) -> str:
        text = "\\boxed{"
        for child in root:
            if child.tag == qn("m:e"):
                text += self.parse(child)
        text += "}"
        return text

    def parse_box(self, root: Element) -> str:
        text = ""
        for child in root:
            text += self.parse(child)
        return text

    def parse_group_chr(self, root: Element) -> str:
        character_map = {
            "←": "\\leftarrow",
            "→": "\\rightarrow",
            "↔": "\\leftrightarrow",
            "⇐": "\\Leftarrow",
            "⇒": "\\Rightarrow",
            "⇔": "\\Leftrightarrow",
        }
        text = "\\underbrace{"
        bottom = False
        for child in root:
            if child.tag == qn("m:groupChrPr"):
                for child2 in child:
                    if child2.tag == qn("m:chr"):
                        char = child2.attrib.get(qn("m:val"))
                        if char in character_map:
                            text = character_map[char]
                for child2 in child:
                    if (
                        child2.tag == qn("m:pos")
                        and child2.attrib.get(qn("m:val")) == "top"
                    ):
                        # If m:pos is set to "top", the symbol is supposed to
                        # be on top and the text is actually supposed to be under
                        bottom = True

        content = ""
        for child in root:
            if child.tag == qn("m:e"):
                content = self.parse(child)
        if text == "\\underbrace{":
            if bottom:
                text = "\\overbrace{" + content + "}"
            else:
                text += content + "}"
        else:
            if not bottom:
                text = "\\overset{" + content + "}" + "{" + text + "}"
            else:
                text = "\\underset{" + content + "}" + "{" + text + "}"
        return text

    def parse_d(self, root: Element) -> str:
        bracket_map = {
            "(": "\\left(",
            ")": "\\right)",
            "[": "\\left[",
            "]": "\\right]",
            "{": "\\left{",
            "}": "\\right}",
            "〈": "\\left\\langle",
            "〉": "\\right\\rangle",
            "⟨": "\\left\\langle",
            "⟩": "\\right\\rangle",
            "⌊": "\\left\\lfloor",
            "⌋": "\\right\\rfloor",
            "⌈": "\\left\\lceil",
            "⌉": "\\right\\rceil",
            "|": "\\left|",
            "‖": "\\left\\|",
            "⟦": "[\\![",
            "⟧": "]\\!]",
        }
        text = ""
        start_bracket = "("
        end_bracket = ")"
        seperator = "|"
        is_matrix = False
        for child in root:
            for child2 in child:
                if child.tag == qn("m:dPr"):
                    if child2.tag == qn("m:begChr"):
                        start_bracket = child2.attrib.get(qn("m:val"))
                    if child2.tag == qn("m:endChr"):
                        end_bracket = child2.attrib.get(qn("m:val"))
                    if child2.tag == qn("m:sepChr"):
                        seperator = child2.attrib.get(qn("m:val"))
                if child2.tag == qn("m:m"):
                    is_matrix = True

        for child in root:
            if child.tag == qn("m:e"):
                if text:
                    text += seperator
                text += self.parse(child)
        end_bracket_replacements = {
            "|": "\\right|",
            "‖": "\\right\\|",
            "[": "\\right[",
        }
        start_bracket_replacements = {
            "]": "\\left]",
        }
        start = ""
        end = ""
        if start_bracket:
            if start_bracket in start_bracket_replacements:
                start = start_bracket_replacements[start_bracket] + " "
            else:
                start = bracket_map[start_bracket] + " "
        if end_bracket:
            if end_bracket in end_bracket_replacements:
                end = " " + end_bracket_replacements[end_bracket]
            else:
                end = " " + bracket_map[end_bracket]
        # If there is no end bracket and this tag contains an m:eqArr tag as a
        # child, we assume that the eqArr should be translated to a cases environment
        # instead of an eqnarray* environment.
        else:
            for child in root:
                if child.tag == qn("m:e"):
                    for child2 in child:
                        if child2.tag == qn("m:eqArr"):
                            text = text.replace("\\begin{eqnarray*}", "")
                            text = text.replace("\\end{eqnarray*}", "")
                            return "\\begin{cases} " + text + " \\end{cases}"
        if is_matrix:
            if start_bracket == "(" and end_bracket == ")":
                return text.replace("{matrix}", "{pmatrix}")
            elif start_bracket == "|" and end_bracket == "|":
                return text.replace("{matrix}", "{vmatrix}")
            elif start_bracket == "‖" and end_bracket == "‖":
                return text.replace("{matrix}", "{Vmatrix}")
            else:
                return text.replace("{matrix}", "{bmatrix}")
        return start + text + end

    def parse_eq_arr(self, root: Element) -> str:
        text = "\\begin{eqnarray*}"
        for child in root:
            if child.tag == qn("m:e"):
                text += self.parse(child) + " \\\\"
        text += "\\end{eqnarray*}"
        return text

    def parse_f(self, root: Element) -> str:
        text = "\\frac{"
        num = ""
        den = ""
        is_binom = False
        for child in root:
            if child.tag == qn("m:fPr"):
                for child2 in child:
                    if (
                        child2.tag == qn("m:type")
                        and child2.attrib.get(qn("m:val")) == "noBar"
                    ):
                        is_binom = True
            if child.tag == qn("m:num"):
                num = self.parse(child)
            if child.tag == qn("m:den"):
                den = self.parse(child)
        if is_binom:
            text = "\\genfrac{}{}{0pt}{}{"
        text += num + "}{" + den + "}"
        return text

    def parse_m(self, root: Element) -> str:
        text = "\\begin{matrix} "
        text += self.parse(root)[:-3]  # Remove the last ' \\'
        text += "\\end{matrix}"
        return text

    def parse_mr(self, root: Element) -> str:
        text = ""
        for child in root:
            if child.tag == qn("m:e"):
                text += self.parse(child) + " & "
        return text[:-2] + "\\\\ "  # Remove the last ' & '

    def parse_func(self, root: Element) -> str:
        function_map = {
            "sin": "\\sin",
            "cos": "\\cos",
            "tan": "\\tan",
            "cot": "\\cot",
            "sec": "\\sec",
            "csc": "\\csc",
            "sinh": "\\sinh",
            "cosh": "\\cosh",
            "tanh": "\\tanh",
            "coth": "\\coth",
            "sech": "\\operatorname{sech}",
            "csch": "\\operatorname{csch}",
            "log": "\\log",
            "ln": "\\ln",
            "min": "\\min",
            "max": "\\max",
            "lim": "\\lim",
        }
        subscript = ""
        superscript = ""
        text = ""
        func_name = "sin"
        for child in root:
            if child.tag == qn("m:fName"):
                for child2 in child:
                    if child2.tag in [qn("m:sSup"), qn("m:sSub"), qn("m:r")]:
                        for child3 in child2:
                            if child3.tag == qn("m:sub"):
                                subscript = self.parse(child3)
                            if child3.tag == qn("m:sup"):
                                superscript = self.parse(child3)
                            if child3.tag == qn("m:t") or child3.tag == qn("m:e"):
                                func_name = self.parse(child3)
                    elif child2.tag == qn("m:limLow"):
                        for child3 in child2:
                            if child3.tag == qn("m:lim"):
                                for child4 in child3:
                                    subscript += self.parse(child4)
                            if child3.tag == qn("m:e"):
                                func_name = self.parse(child3)

            if child.tag == qn("m:e"):
                text += self.parse(child)
        if func_name in ["lim", "max", "min"]:
            return f"\\{func_name}\\limits_{{{subscript}}}^{{{superscript}}}{{{text}}}"
        if func_name not in function_map:
            return f"{{{func_name}}}^{{{superscript}}}_{{{subscript}}}{{{text}}}"
        return function_map[func_name] + f"_{{{subscript}}}^{{{superscript}}}{{{text}}}"

    def parse_s_sup(self, root: Element) -> str:
        content = ""
        exp_content = ""
        for child in root:
            if child.tag == qn("m:e"):
                content = self.parse(child)
            if child.tag == qn("m:sup"):
                exp_content = self.parse(child)
        return f"{{{content}}}^{{{exp_content}}}"

    def parse_s_sub(self, root: Element) -> str:
        content = ""
        sub_content = ""
        for child in root:
            if child.tag == qn("m:e"):
                content = self.parse(child)
            if child.tag == qn("m:sub"):
                sub_content = self.parse(child)
        return f"{{{content}}}_{{{sub_content}}}"

    def parse_s_sub_sup(self, root: Element) -> str:
        content = ""
        sub_content = ""
        exp_content = ""
        for child in root:
            if child.tag == qn("m:e"):
                content = self.parse(child)
            if child.tag == qn("m:sub"):
                sub_content = self.parse(child)
            if child.tag == qn("m:sup"):
                exp_content = self.parse(child)
        return f"{{{content}}}_{{{sub_content}}}^{{{exp_content}}}"

    def parse_s_pre(self, root: Element) -> str:
        content = ""
        sub_content = ""
        exp_content = ""
        for child in root:
            if child.tag == qn("m:e"):
                content = self.parse(child)
            if child.tag == qn("m:sub"):
                sub_content = self.parse(child)
            if child.tag == qn("m:sup"):
                exp_content = self.parse(child)
        return "{}^{" + exp_content + "}_{" + sub_content + "}{" + content + "}"

    def parse_rad(self, root: Element) -> str:
        content = ""
        order = ""
        for child in root:
            if child.tag == qn("m:deg"):
                order = self.parse(child)
            if child.tag == qn("m:e"):
                content += self.parse(child)
        if order:
            return f"\\sqrt[{order}]{{{content}}}"
        return f"\\sqrt{{{content}}}"

    def parse_nary(self, root: Element) -> str:
        character_map = {
            8719: "\\prod",
            8720: "\\coprod",
            8721: "\\sum",
            8747: "\\int",
            8748: "\\iint",
            8749: "\\iiint",
            8750: "\\oint",
            8751: "\\oiint",
            8752: "\\oiiint",
            8896: "\\bigwedge",
            8897: "\\bigvee",
            8898: "\\bigcap",
            8899: "\\bigcup",
        }
        char = 8747
        for child in root:
            if child.tag == qn("m:naryPr"):
                for child2 in child:
                    if child2.tag == qn("m:chr"):
                        char = ord(child2.attrib.get(qn("m:val")))
        text = character_map.get(char, character_map[8721])
        sub = ""
        sup = ""
        content = ""
        for child in root:
            if child.tag == qn("m:sub"):
                sub = self.parse(child)
            if child.tag == qn("m:sup"):
                sup = self.parse(child)
            if child.tag == qn("m:e"):
                content = self.parse(child)
        if sub:
            text += f"_{{{sub}}}"
        if sup:
            text += f"^{{{sup}}}"
        text += "{" + content + "}"
        return text

    parsers = {
        qn("m:r"): parse_r,
        qn("m:acc"): parse_acc,
        qn("m:borderBox"): parse_border_box,
        qn("m:bar"): parse_bar,
        qn("m:box"): parse_box,
        qn("m:d"): parse_d,
        qn("m:e"): parse_e,
        qn("m:groupChr"): parse_group_chr,
        qn("m:f"): parse_f,
        qn("m:sSup"): parse_s_sup,
        qn("m:sSub"): parse_s_sub,
        qn("m:sSubSup"): parse_s_sub_sup,
        qn("m:sPre"): parse_s_pre,
        qn("m:t"): parse_t,
        qn("m:rad"): parse_rad,
        qn("m:nary"): parse_nary,
        qn("m:eqArr"): parse_eq_arr,
        qn("m:func"): parse_func,
        qn("m:m"): parse_m,
        qn("m:mr"): parse_mr,
    }



import zipfile
from defusedxml import ElementTree
from xml.dom import minidom
import re
import os

from docxlatex.parser.utils import linear_expression, qn
from docxlatex.parser import OMMLParser


class Document:
    def __init__(self, document, inline_delimiter="$", block_delimiter="$$"):
        self.document = document
        self.inline_delimiter = inline_delimiter
        self.block_delimiter = block_delimiter
        self.equations = []

    def get_text(
        self,
        linear_format: bool = False,
        get_header_text: bool = False,
        get_footer_text: bool = False,
        image_dir: str | None = None,
        image_extensions: list[str] | tuple[str] | None = None,
    ):
        """
        Extract the text from the .docx file while converting the equations in the document
        to valid LaTeX syntax, enclosed within the specified delimiters

        :param linear_format:bool - True if the inserted equations in the document have been converted
            into linear format in LaTeX syntax, False if the equations are in professional formatting.
            We assume the equations are not in Unicode or any other format. Defaults to False.
        :param get_header_text:bool - True if you want to extract text from the header, False otherwise
        :param get_footer_text:bool - True if you want to extract text from the footer, False otherwise
        :param image_dir:str - The path to the directory where you want to store images
        :param image_extensions:List[str], tuple - A list or tuple of strings of the extensions you want to
            extract (['.jpg', '.png', ...])
        :return text:str - The extracted text
        """
        if image_extensions is None:
            image_extensions = [".jpg", ".jpeg", ".png", ".svg", ".bmp", ".gif"]
        zip_f = zipfile.ZipFile(self.document)
        text = ""
        for f in zip_f.namelist():
            if get_header_text and f.startswith("word/header"):
                text += self.xml_to_text(zip_f.read(f), linear_format)
            if f.startswith("word/document"):
                text += self.xml_to_text(zip_f.read(f), linear_format)
            if get_footer_text and f.startswith("word/footer"):
                text += self.xml_to_text(zip_f.read(f), linear_format)

        if image_dir is not None:
            for f in zip_f.namelist():
                _, extension = os.path.splitext(f)
                if extension in image_extensions:
                    destination = os.path.join(image_dir, os.path.basename(f))
                    with open(destination, "wb") as destination_file:
                        destination_file.write(zip_f.read(f))
        zip_f.close()
        return text

    def get_xml(self):
        """
        :return: The XML representation of the document body (ignores the header and footer)
        """
        _xml = ""
        zip_f = zipfile.ZipFile(self.document)
        for f in zip_f.namelist():
            if f.startswith("word/document"):
                dom = minidom.parse(zip_f.open(f))
                _xml = dom.toprettyxml()
                break
        zip_f.close()
        return _xml

    def pprint_xml(self):
        zip_f = zipfile.ZipFile(self.document)
        for f in zip_f.namelist():
            if f.startswith("word/document"):
                dom = minidom.parse(zip_f.open(f))
                print(dom.toprettyxml())
                break
        zip_f.close()

    def xml_to_text(self, xml: bytes, linear_format: bool) -> str:
        """
        Recursively iterate over the ElementTree of the document and extract text content from supported tags.

        :param xml:str - XML string to be parsed into an xml.etree.Element object.
        :param linear_format:bool - True if the equations in the document have been converted into
        linear format in LaTeX syntax. False if the equations are in professional formatting.
        We assume the equations are not in Unicode or any other format.
        :return text:str - The text contained in the tag
        """
        text = ""
        n_images = 0
        root = ElementTree.fromstring(xml)
        for child in root.iter():
            if child.get("docxlatex_skip_iteration", False):
                continue

            # Found text
            if child.tag == qn("w:t"):
                text += child.text if child.text is not None else ""

            # Found an equation
            elif child.tag == qn("m:oMath"):
                if linear_format:
                    eqn = linear_expression(child)
                    text += self.inline_delimiter + " "
                    text += eqn
                    text += " " + self.inline_delimiter
                    self.equations.append(eqn)
                else:
                    eqn = OMMLParser().parse(child)
                    text += (
                        f"{self.inline_delimiter} " + eqn + f" {self.inline_delimiter}"
                    )
                    self.equations.append(eqn)
            elif child.tag == qn("m:r") and linear_format:
                text += "".join(child.itertext())

            # Found an image
            elif child.tag == qn("w:drawing"):
                n_images += 1
                text += f"\nIMAGE#{n_images}-image{n_images}\n"

            elif child.tag == qn("w:tab"):
                text += "\t"
            elif child.tag == qn("w:br") or child.tag == qn("w:cr"):
                text += "\n"
            elif child.tag == qn("w:p"):
                text += "\n\n"

        text = re.sub(
            rf"\n(\n+){self.inline_delimiter}(\s*.+\s*){self.inline_delimiter}\n",
            rf"\n\1{self.block_delimiter} \2 {self.block_delimiter}",
            text,
        )
        return text



"""
    Utility functions to extract text from the supported mathematical equations from xml tags and
    convert them into LaTeX
"""
from docxlatex.cleaners import clean_exp

ns_map = {
    "w": "http://schemas.openxmlformats.org/wordprocessingml/2006/main",
    "m": "http://schemas.openxmlformats.org/officeDocument/2006/math",
}


def linear_expression(tag):
    """
    Just returns the text contained in the given tag while setting docxlatex_skip_iteration flags
    for all its children.
    :param tag:defusedxml.Element - An xml element which contains a math equation in linear form
    :return text:str - The equation in valid LaTeX syntax
    """
    text = ""
    for child in tag.iter():
        child.set("docxlatex_skip_iteration", True)
        text += child.text if child.text is not None else ""
    text = clean_exp(text)
    return text


def qn(tag):
    """
    A utility function to turn a namespace
    prefixed tag name into a Clark-notation qualified tag name for lxml. For
    example, qn('m:oMath') returns '{http://schemas.openxmlformats.org/officeDocument/2006/math}oMath'

    :param tag:str - A namespace-prefixed tag name
    :return qn:str - A Clark-notation qualified name tag for lxml.
    """
    prefix, tag_root = tag.split(":")
    uri = ns_map[prefix]
    return "{{{}}}{}".format(uri, tag_root)
